@@ -11,7 +11,7 @@ from logging import debug
 from numpy import *
 
 from .core.config import BaseConfig
-from .core.custom_logging import iprint
+from .core.custom_logging import iprint, numpy_set_options
 from .core.Simplex import Simplex
 from .exceptions import ErrorBarError
 from .model import Model
@@ -70,6 +70,7 @@ class DiffEvConfig(BaseConfig):
     use_max_generations:bool=False
     max_generations:int=BaseConfig.GParam(500, pmin=10, pmax=10000, label='Fixed size')
     max_generation_mult:int=BaseConfig.GParam(6, pmin=1, pmax=100, label='Relative size')
+    min_parameter_spread:float=BaseConfig.GParam(0.0, pmin=0.0, pmax=100.0, label='parameter spread to stop (%)')
 
     use_start_guess:bool=True
     use_boundaries:bool=True
@@ -92,7 +93,8 @@ class DiffEvConfig(BaseConfig):
         'Differential Evolution':
             ['km', 'kr', 'create_trial',
              ['Population size:', 'use_pop_mult', 'pop_mult', 'pop_size'],
-             ['Max. Generations:', 'use_max_generations', 'max_generations', 'max_generation_mult']
+             ['Max. Generations:', 'use_max_generations', 'max_generations', 'max_generation_mult'],
+              'min_parameter_spread',
              ],
         'Parallel processing': ['use_parallel_processing', 'parallel_processes', 'parallel_chunksize']
         }
@@ -375,7 +377,7 @@ class DiffEv(GenxOptimizer):
             self.init_fitting(model_obj)
             self.stop=False
             # Start fitting in a new thread
-            threading.Thread(target=self.optimize, daemon=True).start()
+            threading.Thread(target=self.optimize_thread, daemon=True).start()
             self.running=True
             self.text_output('Starting the fit...')
             return True
@@ -404,7 +406,7 @@ class DiffEv(GenxOptimizer):
             self.init_fom_eval()
             n_dim_old=self.n_dim
             if self.n_dim==n_dim_old:
-                threading.Thread(target=self.optimize, daemon=True).start()
+                threading.Thread(target=self.optimize_thread, daemon=True).start()
                 self.text_output('Restarting the fit...')
                 self.running=True
                 return True
@@ -416,6 +418,17 @@ class DiffEv(GenxOptimizer):
             self.text_output('Fit is already running, stop and then start')
             return False
 
+    def optimize_thread(self):
+        try:
+            numpy_set_options()
+            self.optimize()
+        except Exception as e:
+            self.running=False
+            self.error=f'An error occured in the model while executing:\n{e!r}'
+            debug('An error occured in the model while executing', exc_info=True)
+            self.new_best=self.start_guess
+            self.fitting_ended()
+
     def optimize(self):
         """Method that does the optimization.
 
@@ -513,6 +526,14 @@ class DiffEv(GenxOptimizer):
             self.text_output('FOM: %.3f Generation: %d Speed: %.1f'% \
                              (self.best_fom, gen, speed))
 
+            # Check if largest relative spread of a parameter is below the break threshold
+            pop=array(self.pop_vec)
+            norm=(self.par_max-self.par_min)
+            spread=(pop.max(axis=0)-pop.min(axis=0))/norm
+            if spread.max()<(0.01*self.opt.min_parameter_spread):
+                self.text_output('Stopping fit as min_parameter_spread was reached')
+                break
+
             self.new_best=False
             # Do an autosave if activated and the interval is correct
             if gen%self.opt.autosave_interval==0 and self.opt.use_autosave:
@@ -645,6 +666,14 @@ class DiffEv(GenxOptimizer):
                     self.text_output('FOM: %.3f Generation: %d Speed: %.1f'%
                                      (self.best_fom, gen, speed))
 
+                # Check if largest relative spread of a parameter is below the break threshold
+                pop = array(self.pop_vec)
+                norm = (self.par_max-self.par_min)
+                spread = (pop.max(axis=0)-pop.min(axis=0))/norm
+                if spread.max()<(0.01*self.opt.min_parameter_spread):
+                    self.text_output('Stopping fit as min_parameter_spread was reached')
+                    break
+
                 self.new_best=False
                 # Do an autosave if activated and the interval is correct
                 if gen%self.opt.autosave_interval==0 and self.opt.use_autosave:
@@ -1296,12 +1325,17 @@ def parallel_init(model_copy: Model, numba_procs=None, use_mpi=False):
         # ignore KeyboardInterrupt so that master process can handle it
         import signal
         signal.signal(signal.SIGINT, signal.SIG_IGN)
+    seterr(divide='ignore', over='ignore', under='ignore', invalid='ignore')
 
     if numba_procs is not None:
-        import numba
-        if hasattr(numba, 'set_num_threads'):
-            iprint(f"Setting numba threads to {numba_procs}")
-            numba.set_num_threads(numba_procs)
+        try:
+            import numba
+        except ImportError:
+            pass
+        else:
+            if hasattr(numba, 'set_num_threads'):
+                iprint(f"Setting numba threads to {numba_procs}")
+                numba.set_num_threads(numba_procs)
     global model, par_funcs
     model=model_copy
     model.reset()
