@@ -186,7 +186,6 @@ Instrument
     spin flip.
 '''
 
-from genx.gui_logging import iprint
 from .lib import refl
 from .lib import xrmr
 from .lib import edm_slicing as edm
@@ -195,8 +194,6 @@ from .lib import paratt as Paratt
 from .lib import ables as ables
 from .lib import neutron_refl as neutron_refl
 from .lib.physical_constants import muB_to_SL
-from numpy import *
-from scipy.special import erf
 from .lib.instrument import *
 
 mag_limit=1e-8
@@ -1365,26 +1362,18 @@ def slicing_reflectivity(sample, instrument, theta, TwoThetaQz, xray_energy, ret
             n=1-lamda**2*re/pi*(sl_c[:, newaxis]-sl_m1[:, newaxis]*cos(theta*pi/180))/2.0
             Rm=Paratt.Refl_nvary2(theta, lamda*ones(theta.shape), n, d, zeros(d.shape))
             # Hack to check kinematical approx
-            # Rm = slow_paratt.reflq_kin(4 * pi / lamda * sin(theta * pi / 180.), lamda*ones(theta.shape), n, d,
-            #                           zeros(d.shape))
             n=1-lamda**2*re/pi*(sl_c[:, newaxis]+sl_m1[:, newaxis]*cos(theta*pi/180))/2.0
             Rp=Paratt.Refl_nvary2(theta, lamda*ones(theta.shape), n, d, zeros(d.shape))
             # Hack to check kinematical approx
-            # Rp = slow_paratt.reflq_kin(4 * pi / lamda * sin(theta * pi / 180.), lamda * ones(theta.shape), n, d,
-            #                           zeros(d.shape))
             R=(Rp+Rm)/2.0
         elif pol==3 or pol==instrument_string_choices['xpol'][3]:
             # ass
             n=1-lamda**2*re/pi*(sl_c[:, newaxis]-sl_m1[:, newaxis]*cos(theta*pi/180))/2.0
             Rm=Paratt.Refl_nvary2(theta, lamda*ones(theta.shape), n, d, zeros(d.shape))
             # Hack to check kinematical approx
-            # Rm = slow_paratt.reflq_kin(4 * pi / lamda * sin(theta * pi / 180.), lamda * ones(theta.shape), n, d,
-            #                           zeros(d.shape))
             n=1-lamda**2*re/pi*(sl_c[:, newaxis]+sl_m1[:, newaxis]*cos(theta*pi/180))/2.0
             Rp=Paratt.Refl_nvary2(theta, lamda*ones(theta.shape), n, d, zeros(d.shape))
             # Hack to check kinematical approx
-            # Rp = slow_paratt.reflq_kin(4 * pi / lamda * sin(theta * pi / 180.), lamda * ones(theta.shape), n, d,
-            #                           zeros(d.shape))
             R=(Rp-Rm)/(Rp+Rm)
         else:
             raise ValueError('Variable pol has an unvalid value')
