@@ -108,7 +108,7 @@ import numpy as np
 from . import utils
 from .symmetries import SymTrans, Sym
 from .lib.physical_constants import r_e
-from genx.gui_logging import iprint
+from genx.core.custom_logging import iprint
 
 from .lib import USE_NUMBA
 if USE_NUMBA:
@@ -121,7 +121,7 @@ if USE_NUMBA:
 else:
     numba_ss=False
 
-__pars__=['Sample', 'UnitCell', 'Slab', 'AtomGroup', 'Instrument']
+__pars__=['Sample', 'UnitCell', 'Slab', 'AtomGroup', 'Instrument', 'Domain']
 
 class SimMethodInfo:
     def __init__(self, name, args, def_args):
@@ -178,7 +178,7 @@ class Sample:
         return inst.inten*self.calc_f(inst, h, k, l)**2
 
     def calc_f(self, inst, h, k, l):
-        f_list=np.array([domain.calc_f(inst, h, k, l) for domain in self.domains])
+        f_list=np.array([np.sqrt(domain.occ)*domain.calc_f(inst, h, k, l) for domain in self.domains])
         return self._assemble_f_tot(f_list, h, k, l)
 
     def turbo_calc_f(self, inst, h, k, l):
@@ -243,7 +243,7 @@ class Domain:
         if not sym_list:
             sym_list=[SymTrans()]
 
-        if min([type(sym)==type(SymTrans()) for sym in sym_list])==0:
+        if min([sym.__class__.__name__=="SymTrans" for sym in sym_list])==0:
             raise TypeError("All members in the symmetry list has to be a memeber of class SymTrans")
 
         self.surface_sym=sym_list
@@ -261,7 +261,7 @@ class Domain:
         if not sym_list:
             sym_list=[SymTrans()]
 
-        if min([type(sym)==type(SymTrans()) for sym in sym_list])==0:
+        if min([sym.__class__.__name__=="SymTrans" for sym in sym_list])==0:
             raise TypeError("All members in the symmetry list has to be a memeber of class SymTrans")
 
         self.bulk_sym=sym_list
@@ -406,7 +406,14 @@ class Domain:
 
         return x, y, z, u, oc, el
 
-    def create_uc_output(self, use_sym=True, x_uc=1, y_uc=1, fold_sym=True):
+    def _bulk_shifts(self):
+        return self.bulk_slab.dx, self.bulk_slab.dy, self.bulk_slab.dz
+
+    def _surf_shifts(self):
+        dx, dy, dz, c=list(zip(*[(slab.dx, slab.dy, slab.dz, np.ones(len(slab.x))*slab.c) for slab in self.slabs]))
+        return np.r_[dx], np.r_[dy], np.r_[dz]*np.r_[c]
+
+    def create_uc_output(self, use_sym=True, x_uc=1, y_uc=1, fold_sym=True, use_bulk=True):
         ''' Create atomic positions and such for output
 
         Parameters:
@@ -449,19 +456,26 @@ class Domain:
                 oc_sym=np.r_[oc_sym, oc]
                 el_sym=np.r_[el_sym, el]
                 ids_sym.extend(ids)
+        else:
+            x_sym, y_sym, z_sym, u_sym, oc_sym, el_sym, ids_sym=x, y, z, u, oc, el, ids
 
-            for sym_op in self.bulk_sym:
-                if fold_sym:
-                    xb_sym=np.r_[xb_sym, sym_op.trans_x(xb, yb)%1.0]
-                    yb_sym=np.r_[yb_sym, sym_op.trans_y(xb, yb)%1.0]
-                else:
-                    xb_sym=np.r_[xb_sym, sym_op.trans_x(xb, yb)]
-                    yb_sym=np.r_[yb_sym, sym_op.trans_y(xb, yb)]
-                zb_sym=np.r_[zb_sym, zb]
-                ub_sym=np.r_[ub_sym, ub]
-                ocb_sym=np.r_[ocb_sym, ocb]
-                elb_sym=np.r_[elb_sym, elb]
-                idsb_sym.extend(idsb)
+
+        if use_bulk:
+            if use_sym:
+                for sym_op in self.bulk_sym:
+                    if fold_sym:
+                        xb_sym=np.r_[xb_sym, sym_op.trans_x(xb, yb)%1.0]
+                        yb_sym=np.r_[yb_sym, sym_op.trans_y(xb, yb)%1.0]
+                    else:
+                        xb_sym=np.r_[xb_sym, sym_op.trans_x(xb, yb)]
+                        yb_sym=np.r_[yb_sym, sym_op.trans_y(xb, yb)]
+                    zb_sym=np.r_[zb_sym, zb]
+                    ub_sym=np.r_[ub_sym, ub]
+                    ocb_sym=np.r_[ocb_sym, ocb]
+                    elb_sym=np.r_[elb_sym, elb]
+                    idsb_sym.extend(idsb)
+            else:
+                xb_sym, yb_sym, zb_sym, ub_sym, ocb_sym, elb_sym, idsb_sym = xb, yb, zb, ub, ocb, elb, idsb
 
         xout=np.array([])
         yout=np.array([])
@@ -485,6 +499,81 @@ class Domain:
 
         return xout, yout, zout, uout, ocout, elout, idsout
 
+    def create_shift_output(self, use_sym=True, x_uc=1, y_uc=1, fold_sym=True, use_bulk=True):
+        ''' Create atomic shifts (dx,dy,dz) for output similar to create_uc_output '''
+        xb, yb, zb, ub, ocb, elb=self._bulk_pars()
+        dxb, dyb, dzb=self._bulk_shifts()
+        xb0, yb0, zb0=xb-dxb, yb-dyb, zb-dzb
+
+        x, y, z, u, oc, el=self._surf_pars()
+        dx, dy, dz=self._surf_shifts()
+        x0, y0, z0=x-dx, y-dy, z-dz
+
+        dx_sym=np.array([])
+        dy_sym=np.array([])
+        dz_sym=np.array([])
+        dxb_sym=np.array([])
+        dyb_sym=np.array([])
+        dzb_sym=np.array([])
+
+        if use_sym:
+            for sym_op in self.surface_sym:
+                # calculate difference between symmetry atoms and shifted symmetry atoms
+                xt=sym_op.trans_x(x, y)
+                x0t=sym_op.trans_x(x0, y0)
+                yt=sym_op.trans_y(x, y)
+                y0t=sym_op.trans_y(x0, y0)
+
+                dx_sym = np.r_[dx_sym, xt-x0t]
+                dy_sym = np.r_[dy_sym, yt-y0t]
+                dz_sym = np.r_[dz_sym, dz]
+        else:
+            dx_sym, dy_sym, dz_sym=dx, dy, dz
+
+        if use_bulk:
+            if use_sym:
+                for sym_op in self.bulk_sym:
+                    # calculate difference between symmetry atoms and shifted symmetry atoms
+                    xt = sym_op.trans_x(xb, yb)
+                    x0t = sym_op.trans_x(xb0, yb0)
+                    yt = sym_op.trans_y(xb, yb)
+                    y0t = sym_op.trans_y(xb0, yb0)
+
+                    dxb_sym = np.r_[dxb_sym, xt-x0t]
+                    dyb_sym = np.r_[dyb_sym, yt-y0t]
+                    dzb_sym = np.r_[dzb_sym, dzb]
+            else:
+                dxb_sym, dyb_sym, dzb_sym = dxb, dyb, dzb
+
+        dxout=np.array([])
+        dyout=np.array([])
+        dzout=np.array([])
+        for i in range(x_uc):
+            for j in range(y_uc):
+                dxout=np.r_[dxout, dx_sym, dxb_sym]
+                dyout=np.r_[dyout, dy_sym, dyb_sym]
+                dzout=np.r_[dzout, dz_sym, dzb_sym]
+        # Create output in cartesian coordinates:
+        dxout, dyout, dzout=self.unit_cell.cart_coords(dxout, dyout, dzout)
+
+        return dxout, dyout, dzout
+
+    def export_xyz(self, fname, use_sym=True, x_uc=1, y_uc=1, fold_sym=True, use_bulk=False):
+        from genx.version import __version__ as version
+        if not fname.endswith('.xyz'):
+            fname+='.xyz'
+        x, y, z, u, oc, el, ids = self.create_uc_output(use_sym=use_sym, x_uc=x_uc, y_uc=y_uc,
+                                                        fold_sym=fold_sym, use_bulk=use_bulk)
+        uc_a=self.unit_cell.a
+        uc_b=self.unit_cell.b
+        uc_c=self.unit_cell.c
+        c_total=uc_c*sum([sl.c for sl in self.slabs])
+        with open(fname, 'w') as fh:
+            fh.write(f'{len(x)}\n')
+            fh.write(f'# structure exported by GenX {version}, UC: a={uc_a} b={uc_b} c={c_total}\n')
+            for xi, yi, zi, eli in zip(x,y,z,el):
+                fh.write(f'{eli:3s} {xi:-12.7f} {yi:-12.7f} {zi:-12.7f}\n')
+
     def _get_f(self, inst, el, dinv):
         '''from the elements extract an array with atomic structure factors
         '''
