@@ -48,7 +48,7 @@ class Plugin(Template):
         self.skip_rows=23
         self.delimiter=None
 
-    def LoadData(self, dataset, filename):
+    def LoadData(self, dataset, filename, data_id=0):
         '''LoadData(self, data_item_number, filename) --> none
         
         Loads the data from filename into the data_item_number.
